//
//  AppDelegate.swift
//  Timer
//
//  Designed by Lech Szymanski in 2014.
//  Swift implementation by David Eyers in 2015.
//  Updated for Swift 4.x / Xcode 9.4 in 2018 by Hamza Bennani.
//
//

import Cocoa

@NSApplicationMain
class AppDelegate: NSObject, NSApplicationDelegate {

    @IBOutlet weak var window: NSWindow!
    
    // Outlets to the menus that we added
    @IBOutlet weak var startMenuItem: NSMenuItem!
    @IBOutlet weak var stopMenuItem: NSMenuItem!

    var timerWindows = [TimerWindowController]()
    
    // Reference to the preferences panel window
    let preferencesWindow = PreferencesController(windowNibName: NSNib.Name(rawValue: "PreferencesPanel"))

    override init() {
        super.init()
        // If AppDelegate weren't a singleton, would need to use a Swift dispatch_once block
        let sToTTransformer = SecondsToTimerTransformer()
        let rToTTransformer = RunningToTitleTransformer()

        ValueTransformer.setValueTransformer(sToTTransformer, forName: NSValueTransformerName(rawValue: "SecondsToTimerTransformer"))
        ValueTransformer.setValueTransformer(rToTTransformer, forName: NSValueTransformerName(rawValue: "RunningToTitleTransformer"))
        
        // Initialise the preferences factory defaults (once)
        let backColor:Data = NSKeyedArchiver.archivedData(withRootObject: NSColor.windowBackgroundColor)
        let fontColor:Data = NSKeyedArchiver.archivedData(withRootObject: NSColor.black)
        let bttnsVisible = NSNumber(value: true as Bool)
        
        let defaultValues = [TimerBackColorKey:backColor, TimerFontColorKey:fontColor, TimerBttnsVisibleKey:bttnsVisible] as [String : Any]
        UserDefaults.standard.register(defaults: defaultValues)
    }
    
    func applicationDidFinishLaunching(_ aNotification: Notification) {
        // Register ourselves for notification when windows close
        let nc = NotificationCenter.default
        nc.addObserver(self, selector: #selector(AppDelegate.windowClosingHandler(_:)), name: NSWindow.willCloseNotification, object: nil)
        
        newTimerWindow(self)
    }
    
    @objc func windowClosingHandler(_ note:Notification){
        let winToClose:NSWindow = note.object as! NSWindow
        for (index,ctrl) in timerWindows.enumerated() {
            if ctrl.window == winToClose {
                timerWindows.remove(at: index)
                break
            }
        }
        if timerWindows.count == 0 {
            NSApplication.shared.terminate(self)
        }
    }

    func applicationWillTerminate(_ aNotification: Notification) {
        // Insert code here to tear down your application
    }

    @IBAction func newTimerWindow(_ sender: AnyObject) {
        let timerWindow = TimerWindowController()
        timerWindows.append(timerWindow)
        timerWindow.showWindow(self)
    }

    @IBAction func openPreferencesPanel(_ sender: AnyObject) {
        preferencesWindow.showWindow(self)
    }
}
