//
//  PreferencesController.swift
//  Timer
//
//  Designed by Lech Szymanski in 2014.
//  Swift implementation by David Eyers in 2015.
//  Updated for Swift 4.x / Xcode 9.4 in 2018 by Hamza Bennani.
//
import Cocoa

// Define archive keys for different settings
let TimerBackColorKey = "TimerBackColor"
let TimerFontColorKey = "TimerFontColor"
let TimerBttnsVisibleKey = "TimerBttnsVisible"

class PreferencesController: NSWindowController {

    @IBOutlet weak var backColorWell: NSColorWell!
    @IBOutlet weak var fontColorWell: NSColorWell!
    @IBOutlet weak var visibilityCheck: NSButton!
    
    override func windowDidLoad() {
        super.windowDidLoad()
        
        backColorWell.color = PreferencesController.preferencesBackColor()
        fontColorWell.color = PreferencesController.preferencesFontColor()
        if PreferencesController.preferencesButtonsVisible() {
           visibilityCheck.state = NSControl.StateValue(rawValue: 1)
        } else {
            visibilityCheck.state = NSControl.StateValue(rawValue: 0)
        }
    }

    // Actions for elements on the preference window
    @IBAction override func changeColor(_ s: Any?) {
        let sender = s as! NSObject
        if sender == backColorWell {
            PreferencesController.setPreferencesBackColor(backColorWell.color)
        } else if sender == fontColorWell {
            PreferencesController.setPreferencesFontColor(fontColorWell.color)
        }
    }
    
    @IBAction func changeVisibility(_ sender: AnyObject) {
        PreferencesController.setPreferencesButtonsVisible(visibilityCheck.state.rawValue==1)
    }
    
    class func preferenceColor(_ keyStr:String)->NSColor {
        // Read the colour setting from defaults
        let defaults = UserDefaults.standard
        let color:Data = defaults.object(forKey: keyStr) as! Data
        return NSKeyedUnarchiver.unarchiveObject(with: color) as! NSColor
    }
    
    class func preferencesBackColor()->NSColor {
        return preferenceColor(TimerBackColorKey)
    }
    class func preferencesFontColor()->NSColor {
        return preferenceColor(TimerFontColorKey)
    }
    class func preferencesButtonsVisible()->Bool {
        let defaults = UserDefaults.standard
        return defaults.bool(forKey: TimerBttnsVisibleKey)
    }
    class func setPreferencesColor(_ color:NSColor, withKey keyStr:String){
        // Save colour to the defaults
        let colorAsData:Data = NSKeyedArchiver.archivedData(withRootObject: color)
        let defaults = UserDefaults.standard
        defaults.set(colorAsData, forKey: keyStr)
        
        // Post a notification about the preference change
        let d = ["color":color]
        let nc = NotificationCenter.default
        nc.post(name: Notification.Name(rawValue: keyStr), object: self, userInfo: d)
    }
    class func setPreferencesBackColor(_ color:NSColor) {
        setPreferencesColor(color, withKey: TimerBackColorKey)
    }
    class func setPreferencesFontColor(_ color:NSColor) {
        setPreferencesColor(color, withKey: TimerFontColorKey)
    }
    class func setPreferencesButtonsVisible(_ visible:Bool) {
        let defaults = UserDefaults.standard
        defaults.set(visible, forKey: TimerBttnsVisibleKey)
        
        // Post a notification about the preference change
        let d = ["buttons":visible]
        let nc = NotificationCenter.default
        nc.post(name: Notification.Name(rawValue: TimerBttnsVisibleKey), object: self, userInfo: d)
    }
}
