//
//  TimerModel.swift
//  Timer
//
//  Designed by Lech Szymanski in 2014.
//  Swift implementation by David Eyers in 2015.
//  Updated for Swift 4.x / Xcode 9.4 in 2018 by Hamza Bennani.
//

import Foundation

open class TimerModel: NSObject {
    @objc   dynamic var seconds:Int = 0
    var stopping:Bool = false
    
    @objc   open dynamic var running:Bool = false
    
    // Create an undomanager for this timer
    let undomngr = UndoManager()
    
    // the OS's NSTimer will callback here
    @objc func countUp(_ theTimer:Foundation.Timer) {
        if(!stopping) {
            seconds += 1
        } else {
            theTimer.invalidate()
            running = false
            stopping = false
        }
    }
    
    open func start() {
        if(running) { return }
        
        // Create a timer object that calls the countUp method every second
        let timer = Foundation.Timer(timeInterval: 1, target: self, selector: #selector(TimerModel.countUp(_:)), userInfo: nil, repeats: true)
        
        // Get a reference to the current event loop
        let loop = RunLoop.current
        
        // Attach the above timer to the event loop:
        // it will then actually start firning
        loop.add(timer, forMode: RunLoopMode.commonModes)
        
        // We'll say the opposite of starting a timer is stopping it at a particular count of seconds
        undomngr.setActionName("Start at \(seconds)")
        (undomngr.prepare(withInvocationTarget: self) as AnyObject).stopAtSeconds(seconds)
        
        // Change our internal state to running
        running = true
    }
    open func stop() {
        // We'll say the opposite of stopping a timer is starting it at a particular count of seconds
        undomngr.setActionName("Stop at \(seconds)")
        (undomngr.prepare(withInvocationTarget: self) as AnyObject).startAtSeconds(seconds)

        stopping = true
    }
    open func reset() {
        // We'll say the opposite of resetting a timer is stopping it at a particular count of seconds
        undomngr.setActionName("Reset")
        (undomngr.prepare(withInvocationTarget: self) as AnyObject).stopAtSeconds(seconds)

        seconds = 0
    }
    @objc open func startAtSeconds(_ s:Int) {
        seconds = s
        start()
    }
    @objc open func stopAtSeconds(_ s:Int) {
        stop()
        seconds = s
    }
}
